package creational.singleton;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

// 懒汉式，线程安全

public class LazyThread implements Runnable{


    private static int POOL_NUM = 5;
    private static LazyThread instance;
    private LazyThread (){}
    public static synchronized LazyThread getInstance() throws InterruptedException {
        //这里是粗暴地对整个 getInstance() 方法加锁，这样做代价很大，
        // 因为，只有当第一次调用 getInstance() 时才需要同步创建对象，创建之后再次调用 getInstance()
        // 时就只是简单的返回成员变量，而这里是无需同步的，所以没必要对整个方法加锁。
        //
        //由于同步一个方法会降低上百倍甚至更高的性能， 每次调用获取和释放锁的开销似乎是可以避免的：
        // 一旦初始化完成，获取和释放锁就显得很不必要。
        if (instance == null) {
            instance = new LazyThread();
            System.out.println();
        }
        return instance;
    }
    @Override
    public void run() {
        try {
            System.out.println(LazyThread.getInstance());
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

    public static void main(String[] args) throws InterruptedException {
        ExecutorService executorService = Executors.newFixedThreadPool(3);
        for (int i = 0; i < POOL_NUM; i++) {
            LazyThread thread = new LazyThread();
            Thread.sleep(1000);
            executorService.execute(thread);
        }
        executorService.shutdown();
    }
}
